"""Annotations are used to provide additional metadata about resources,
class instances and other datastructures.
"""
#pylint: disable=bad-whitespace


DJANGO_DOMAIN               = 'django.python.unimatrixone.io'
DJANGO_PROJECT_NAME         = '%s/project' % DJANGO_DOMAIN
DJANGO_SETTINGS_MODULE      = '%s/settings' % DJANGO_DOMAIN
PLATFORM_DOMAIN             = "platform.unimatrixone.io"
PLATFORM_LANGUAGE_NAME      = "%s/language-name" % PLATFORM_DOMAIN
PLATFORM_LANGUAGE_VERSION   = "%s/language-version" % PLATFORM_DOMAIN
PLATFORM_FRAMEWORK_NAME     = "%s/framework-name" % PLATFORM_DOMAIN
PLATFORM_FRAMEWORK_VERSION  = "%s/framework-version" % PLATFORM_DOMAIN
PLATFORM_OS_NAME            = "%s/os-name" % PLATFORM_DOMAIN
PLATFORM_OS_RELEASE         = "%s/os-release" % PLATFORM_DOMAIN
